/*
 * libdigidocpp
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#pragma once

#include "Exports.h"

#include <string>
#include <vector>

namespace digidoc
{
    /**
     * Base exception class of the digidoc implementation. Digidoc library
     * should never throw exceptions other type than instance of this class
     * or class inherited from this class like IOException or BDocException.
     *
     * @author Janari Põld
     */
    class EXP_DIGIDOC Exception
    {
      public:
          /**
           * Exception code
           */
          enum ExceptionCode {
              CertificateIssuerMissing = 10,
              CertificateRevoked       = 5,
              CertificateUnknown       = 6,
              NoException              = 0,
              OCSPResponderMissing     = 8,
              OCSPCertMissing          = 9,
              OCSPTimeSlot             = 7,
              OCSPRequestUnauthorized  = 11,
              PINCanceled              = 2,
              PINFailed                = 4,
              PINIncorrect             = 1,
              PINLocked                = 3,
          };
          typedef std::vector<Exception> Causes;

          Exception(const std::string& file, int line, const std::string& msg);
          Exception(const std::string& file, int line, const std::string& msg, const Exception& cause);
          ExceptionCode code() const;
          int ddoc() const;
          std::string getMsg() const;
          bool hasCause() const;
          Causes getCauses() const;
          void addCause(const Exception& cause);
          void setCode( ExceptionCode Code );
          void setDDoc( int err );

      protected:
          std::string file;
          std::string msg;
          int line;
          int m_ddoc;
          Causes causes;
          ExceptionCode m_code;
    };

    /**
     * BDOC exception implementation. Thrown if the BDOC is in incorrect format or
     * is used incorrectly.
     *
     * @author Janari Põld
     */
    class EXP_DIGIDOC BDocException : public Exception
    {
      public:
          BDocException(const std::string& file, int line, const std::string& msg);
          BDocException(const std::string& file, int line, const std::string& msg, const Exception& cause);
    };

    /**
     * IO exception implementation. Thrown if IO operations failed.
     *
     * @author Janari Põld
     */
    class EXP_DIGIDOC IOException : public Exception
    {
      public:
          IOException(const std::string& file, int line, const std::string& msg);
          IOException(const std::string& file, int line, const std::string& msg, const Exception& cause);
    };

    /**
     * Sign exception implementation. Thrown if the sign operation fails
     *
     * @author Janari Põld
     */
    class EXP_DIGIDOC SignException : public Exception
    {
      public:
          SignException(const std::string& file, int line, const std::string& msg);
          SignException(const std::string& file, int line, const std::string& msg, const Exception& cause);
    };

    /**
     * Signature exception implementation. Thrown if the signature is in incorrect format or
     * is used incorrectly.
     *
     * @author Janari Põld
     */
    class EXP_DIGIDOC SignatureException : public Exception
    {
      public:
          SignatureException(const std::string& file, int line, const std::string& msg);
          SignatureException(const std::string& file, int line, const std::string& msg, const Exception& cause);
    };
}
